-- Player BaseClass

-- The whole point of this structure is to focus on one place at a time. 
-- This file will load the things the player needs. 

	local idleSheet = lg.newImage("idle.png")
	local walkSheet = lg.newImage("walk.png")
	local crouchSheet = lg.newImage("crouch.png")	
	local grabSheet = lg.newImage("grab.png")	
	local jumpSheet = lg.newImage("jump.png")	
	local landSheet = lg.newImage("land.png")	
	
	local xanIdle = newAnimation( idleSheet, 128, 128, 0.1, 0)
	local xanWalk = newAnimation( walkSheet, 150, 128, 0.1, 7)
	local xanCrouch = newAnimation( crouchSheet, 128, 128, 0.2, 0)
	local xanGrab = newAnimation( grabSheet, 150, 150, 0.1, 0)
	local xanJump = newAnimation( jumpSheet, 128, 180, 0.2, 0)
	local xanLand = newAnimation( landSheet, 128, 180, 0.25, 4)
	--Set Animation Default modes. ( loop is default ) 


xan = {}
	

xan = {     
		pos = {	
				x = lg.getWidth()/5 , -- based on screen size
				y = lg.getHeight()/2 
			  },
				
		drawVars = {				
				rot = 0, -- draw vars
				sx = 1,
				sy = 1,
				xo = 64,
				yo = 115,
				   },
		
		speed = 190, --Config Vars (speed == velX, jspd = velY .... REMEMBR IT!
		jspd = 0, -- to be incremented by accel. and decremented by gravity. 
		jsmx = 30, -- the maximum speed of jumps
		jhgt = 280,
		jacl = 10,
		
		state = "idle", --Default States
		facing = "right",
		grab = false, -- is grabbing
		jump = false, -- is jumping
		canJump = false, -- Xan can only jump when his feet are on the ground. 
		

		}



function xan:update(dt)

	local mx = love.mouse.getX() -- mouse position
	local my = love.mouse.getY()

-- if player hits rock then lets take rock out of the world table. 
--[[
	if checkHit(xan,rock) then
		table.remove(world,rock)
	end

--]]

--First Evaluate States
	if blocked == false then 
		xan.state = "walk"
		xan.pos.x = xan.pos.x +(xan.drawVars.sx * xan.speed * dt) -- move xan. 
	elseif love.mouse.isDown('r') then 
		xan.state = "grab"
	elseif blocked then -- right now blocked means 'hitting' mouse. Lets hit the rock after this jump bit. 
		xan.state = "idle" 

	end
	

	if love.mouse.isDown('l') and xan.canJump then
		xan.Jump() 
	end 


	
	
--Crouch Event -- comes after to interrupt. Could be added condition?
	if xan.pos.y < my and blocked == true then
		xan.state = "crouch"
	end
	

--update Facing Scale
	if xan.pos.x > mx then -- mouse is left of xan. 
		xan.drawVars.sx = -1 -- xan is facing left. 
		xan.facing = "left"
	elseif xan.pos.x < mx then 
		xan.drawVars.sx = 1
		xan.facing = "right"
	end
	

	if xan.pos.y < groundLevel and xan.jump == false then  -- change to ground collision in future. 
		-- update gravity. "pull" the player down. 
		xan.pos.y = xan.pos.y + grav
		xan.jspd = xan.jspd - grav -- part of the jump hack problem. 
		xan.state = "land"
	end
	
	if xan.pos.y >= groundLevel then -- more hacks. Lets have a ground state in here
		xan.canJump = true
	end
	
	-- cleanup Values. 
	if xan.jspd <= 0 then -- hack. Will need cleanup by midterm. 
		xan.jspd = 0
	end
	
	if xan.pos.y > groundLevel then
		xan.pos.y = groundLevel
	end

		
-- animation State handling ( Switch Animations )
	if xan.state == "idle" then 
		xanIdle:update(dt) -- updates animation
	elseif xan.state == "walk" then
		xanWalk:update(dt) 
	elseif xan.state == "crouch" then
		xanCrouch:update(dt) 
	elseif xan.state == "grab" then
		xanGrab:update(dt) 
	elseif xan.state == "jump" then
		xanJump:update(dt) 
	elseif xan.state == "land" then
		xanLand:update(dt) 
	end

	
-- animation constraints (Similar to collision)
	if mx >= xan.pos.x - xan.drawVars.xo and mx <= xan.pos.x + xan.drawVars.xo then 
		xan.pos.x = xan.pos.x
		blocked = true
	else 
		blocked = false
	end
	

	

end 

function xan:draw() -- changes the animation depending on the state. 
	
	if xan.state == "crouch" then
		xanCrouch:draw(xan.pos.x, xan.pos.y, xan.drawVars.rot, 
					 xan.drawVars.sx, xan.drawVars.sy, xan.drawVars.xo, xan.drawVars.yo)
	end
	
	if xan.state == "grab" then
		xanGrab:draw(xan.pos.x, xan.pos.y, xan.drawVars.rot, 
					 xan.drawVars.sx, xan.drawVars.sy, xan.drawVars.xo, xan.drawVars.yo)
	end
	
	if xan.state == "idle" then    --- this section actually is working with the animation switches. 
		xanIdle:draw(xan.pos.x, xan.pos.y, xan.drawVars.rot, 
					 xan.drawVars.sx, xan.drawVars.sy, xan.drawVars.xo, xan.drawVars.yo)
	end
	
	if xan.state == "walk" then
		xanWalk:draw(xan.pos.x, xan.pos.y, xan.drawVars.rot, 
					 xan.drawVars.sx, xan.drawVars.sy, xan.drawVars.xo, xan.drawVars.yo)
	end

	if xan.state == "jump" then
		xanJump:draw(xan.pos.x, xan.pos.y, xan.drawVars.rot, 
					 xan.drawVars.sx, xan.drawVars.sy, xan.drawVars.xo, xan.drawVars.yo + 30)
	end	
	
	if xan.state == "land" then
		xanLand:draw(xan.pos.x, xan.pos.y, xan.drawVars.rot, 
					 xan.drawVars.sx, xan.drawVars.sy, xan.drawVars.xo, xan.drawVars.yo + 30) -- adjust for image size 
	end

	
	--debug to show origin
	love.graphics.circle("fill", xan.pos.x, xan.pos.y, 5)
	
end 

function xan:Jump() -- remember the Capitol methods. Jump,Slam,Bite.. what have you. 

	xan.state = "jump"
	xan.jump = true
	local jmax = groundLevel - xan.jhgt -- will need last ground in future. For landing on rocks.
	if (xan.pos.y - xan.jspd) <= jmax then
		xan.canJump = false
		xan.jump = false
	end
	if xan.canJump then
		xan.jspd = xan.jspd + xan.jacl 
		if xan.jspd >= xan.jsmx then 
			xan.jspd = xan.jsmx -- limit the jump speed 
		end
	end

	
	
	xan.pos.y = xan.pos.y - xan.jspd	

end 

function love.mousereleased(x, y, btn)
	
	if btn == 'l' then 
		xan.jump = false
		xan.canJump = false
		xanLand:seek(1)
		xanJump:seek(1) -- we seek the animation so it always starts at the beginning when we release the mouse.
	end
	
end 


-- maybe just an abstract jist of collision will work? 
	
-- checkHit(xan,rock) -- (a,b)
--[[
if xan is on the left of the object
or if the object is to the left of xan ( xan.pos.x - xan.drawVars.xo > b.pos.x + b.drawVars.xo)
												left border       greater than    right border
or if the a.topBorder is > b	

return false ( if xan is outside of the other object. 


then can we say in:player on Update(dt) checkHit(rock)
											for all the rocks in the world.
											if I hit a rock and state is grab then on frame 4 of animation
											hold the animation. Else. Loop and loose stamina. 
											
											
		
--]]





